#include "All.h"
#include "BufferIO.h"

namespace APE
{

CBufferIO::CBufferIO(CIO * pSource, int nBufferBytes)
{
    m_pSource = pSource;
    m_nBufferBytes = 0;
    m_nBufferTotalBytes = nBufferBytes;
    m_spBuffer.Assign(new unsigned char [size_t(m_nBufferTotalBytes)], true);
    m_bReadToBuffer = true;
}

CBufferIO::~CBufferIO()
{
    Close();
    SAFE_DELETE(m_pSource);
}

int CBufferIO::Open(const wchar_t * pName, bool bOpenReadOnly)
{
    return m_pSource->Open(pName, bOpenReadOnly);
}

int CBufferIO::Close()
{
    return m_pSource->Close();
}
    
int CBufferIO::Read(void * pBuffer, unsigned int nBytesToRead, unsigned int * pBytesRead)
{
    int nResult = m_pSource->Read(pBuffer, nBytesToRead, pBytesRead);

    if (m_bReadToBuffer && (m_spBuffer != NULL) && (*pBytesRead > 0))
    {
        int nBufferBytes = ape_min(m_nBufferTotalBytes - m_nBufferBytes, *(int *)pBytesRead);
        if (nBufferBytes <= 0)
        {
            m_bReadToBuffer = false;
        }
        else
        {
            memcpy(&m_spBuffer[m_nBufferBytes], pBuffer, size_t(nBufferBytes));
            m_nBufferBytes += *pBytesRead;
        }
    }

    return nResult;
}

int CBufferIO::Write(const void * pBuffer, unsigned int nBytesToWrite, unsigned int * pBytesWritten)
{
    return m_pSource->Write(pBuffer, nBytesToWrite, pBytesWritten);
}

int64 CBufferIO::PerformSeek()
{
    m_bReadToBuffer = false; // we seeked, so stop buffering
    return m_pSource->PerformSeek();
}

int CBufferIO::SetEOF()
{
    return m_pSource->SetEOF();
}

unsigned char * CBufferIO::GetBuffer(int * pnBufferBytes)
{
    if (*pnBufferBytes > m_nBufferTotalBytes)
        return NULL; // the request exceeded the size we stored, so return NULL
    ASSERT(*pnBufferBytes == m_nBufferBytes);
    *pnBufferBytes = m_nBufferBytes;
    m_bReadToBuffer = false; // no longer needed
    return m_spBuffer;
}

int64 CBufferIO::GetPosition()
{
    if (m_bReadToBuffer)
    {
        // getting the position on a pipe doesn't work, so we need to do this
        return m_nBufferBytes;
    }
    else
    {
        return m_pSource->GetPosition();
    }
}

int64 CBufferIO::GetSize()
{
    return m_pSource->GetSize();
}

int CBufferIO::GetName(wchar_t * pBuffer)
{
    return m_pSource->GetName(pBuffer);
}

int CBufferIO::Create(const wchar_t * pName)
{
    return m_pSource->Create(pName);
}

int CBufferIO::Delete()
{
    return m_pSource->Delete();
}

}
